@@ -2,7 +2,10 @@ use crate::algorithm::bulk_load;
 use crate::algorithm::intersection_iterator::IntersectionIterator;
 use crate::algorithm::iterators::*;
 use crate::algorithm::nearest_neighbor;
+use crate::algorithm::nearest_neighbor::NearestNeighborDistance2Iterator;
+use crate::algorithm::nearest_neighbor::NearestNeighborIterator;
 use crate::algorithm::removal;
+use crate::algorithm::removal::DrainIterator;
 use crate::algorithm::selection_functions::*;
 use crate::envelope::Envelope;
 use crate::node::ParentNode;
@@ -251,6 +254,10 @@ where
         self.size
     }
 
+    pub(crate) fn size_mut(&mut self) -> &mut usize {
+        &mut self.size
+    }
+
     /// Returns an iterator over all elements contained in the tree.
     ///
     /// The order in which the elements are returned is not specified.
@@ -258,7 +265,7 @@ where
     /// # Example
     /// ```
     /// use rstar::RTree;
-    /// let tree = RTree::bulk_load(vec![[0.0, 0.1], [0.3, 0.2], [0.4, 0.2]]);
+    /// let tree = RTree::bulk_load(vec![(0.0, 0.1), (0.3, 0.2), (0.4, 0.2)]);
     /// for point in tree.iter() {
     ///     println!("This tree contains point {:?}", point);
     /// }
@@ -310,6 +317,12 @@ where
         LocateInEnvelopeMut::new(&mut self.root, SelectInEnvelopeFunction::new(*envelope))
     }
 
+    /// Draining variant of [locate_in_envelope](#method.locate_in_envelope).
+    pub fn drain_in_envelope(&mut self, envelope: T::Envelope) -> DrainIterator<T, SelectInEnvelopeFunction<T>, Params> {
+        let sel = SelectInEnvelopeFunction::new(envelope);
+        self.drain_with_selection_function(sel)
+    }
+
     /// Returns all elements whose envelope intersects a given envelope.
     ///
     /// Any element fully contained within an envelope is also returned by this method. Two
@@ -375,7 +388,7 @@ where
     pub fn locate_with_selection_function<S: SelectionFunction<T>>(
         &self,
         selection_function: S,
-    ) -> impl Iterator<Item = &T> {
+    ) -> SelectionIterator<T, S> {
         SelectionIterator::new(&self.root, selection_function)
     }
 
@@ -383,7 +396,7 @@ where
     pub fn locate_with_selection_function_mut<S: SelectionFunction<T>>(
         &mut self,
         selection_function: S,
-    ) -> impl Iterator<Item = &mut T> {
+    ) -> SelectionIteratorMut<T, S> {
         SelectionIteratorMut::new(&mut self.root, selection_function)
     }
 
@@ -441,11 +454,35 @@ where
     where
         F: SelectionFunction<T>,
     {
-        let result = removal::remove::<_, Params, _>(&mut self.root, &function);
-        if result.is_some() {
-            self.size -= 1;
-        }
-        result
+        removal::DrainIterator::new(self, function).take(1).last()
+    }
+
+    /// Drain elements selected by a [`SelectionFunction`]. Returns an
+    /// iterator that successively removes selected elements and returns
+    /// them. This is the most generic drain API, see also:
+    /// [`RTree::drain_in_envelope_intersecting`],
+    /// [`RTree::drain_within_distance`].
+    ///
+    /// # Remarks
+    ///
+    /// This API is similar to `Vec::drain_filter`, but stopping the
+    /// iteration would stop the removal. However, the returned iterator
+    /// must be properly dropped. Leaking this iterator leads to a leak
+    /// amplification, where all the elements in the tree are leaked.
+    pub fn drain_with_selection_function<F>(&mut self, function: F) -> DrainIterator<T, F, Params>
+    where
+        F: SelectionFunction<T>,
+    {
+        removal::DrainIterator::new(self, function)
+    }
+
+    /// Drains elements intersecting the `envelope`. Similar to
+    /// `locate_in_envelope_intersecting`, except the elements are removed
+    /// and returned via an iterator.
+    pub fn drain_in_envelope_intersecting(&mut self, envelope: T::Envelope) -> DrainIterator<T, SelectInEnvelopeFuncIntersecting<T>, Params>
+    {
+        let selection_function = SelectInEnvelopeFuncIntersecting::new(envelope);
+        self.drain_with_selection_function(selection_function)
     }
 }
 
@@ -651,6 +688,19 @@ where
         LocateWithinDistanceIterator::new(self.root(), selection_function)
     }
 
+    /// Drain all elements of the tree within a certain distance.
+    ///
+    /// Similar to [`RTree::locate_within_distance`], but removes and
+    /// returns the elements via an iterator.
+    pub fn drain_within_distance(
+        &mut self,
+        query_point: <T::Envelope as Envelope>::Point,
+        max_squared_radius: <<T::Envelope as Envelope>::Point as Point>::Scalar,
+    ) -> DrainIterator<T, SelectWithinDistanceFunction<T>, Params> {
+        let selection_function = SelectWithinDistanceFunction::new(query_point, max_squared_radius);
+        self.drain_with_selection_function(selection_function)
+    }
+
     /// Returns all elements of the tree sorted by their distance to a given point.
     ///
     /// # Runtime
@@ -673,7 +723,7 @@ where
     pub fn nearest_neighbor_iter(
         &self,
         query_point: &<T::Envelope as Envelope>::Point,
-    ) -> impl Iterator<Item = &T> {
+    ) -> NearestNeighborIterator<T> {
         nearest_neighbor::NearestNeighborIterator::new(&self.root, *query_point)
     }
 
@@ -685,7 +735,7 @@ where
     pub fn nearest_neighbor_iter_with_distance(
         &self,
         query_point: &<T::Envelope as Envelope>::Point,
-    ) -> impl Iterator<Item = (&T, <<T::Envelope as Envelope>::Point as Point>::Scalar)> {
+    ) -> NearestNeighborDistance2Iterator<T> {
         nearest_neighbor::NearestNeighborDistance2Iterator::new(&self.root, *query_point)
     }
 
@@ -696,7 +746,7 @@ where
     pub fn nearest_neighbor_iter_with_distance_2(
         &self,
         query_point: &<T::Envelope as Envelope>::Point,
-    ) -> impl Iterator<Item = (&T, <<T::Envelope as Envelope>::Point as Point>::Scalar)> {
+    ) -> NearestNeighborDistance2Iterator<T> {
         nearest_neighbor::NearestNeighborDistance2Iterator::new(&self.root, *query_point)
     }
 
